import { Page } from "@/pages/base";
import React, { useEffect, useState } from "react";
import { useTranslation } from "react-i18next";
import { useAsyncState } from "@/hooks";
import { InternshipType } from "@/data";
import api from "@/management/api";
import { Management } from "@/management/main";
import { Button, Container, Tooltip, Typography } from "@material-ui/core";
import { Async } from "@/components/async";
import MaterialTable, { Column } from "material-table";
import { MaterialTableTitle } from "@/management/common/MaterialTableTitle";
import { actionsColumn, fieldComparator, multilingualStringComparator } from "@/management/common/helpers";
import { AccountCheck, Delete, Refresh, ShieldCheck } from "mdi-material-ui";
import { OneOrMany } from "@/helpers";
import { createDeleteAction } from "@/management/common/DeleteResourceAction";
import { BulkActions } from "@/management/common/BulkActions";
import { useSpacing } from "@/styles";
import { Actions } from "@/components";
import { MultilingualCell } from "@/management/common/MultilangualCell";

const title = "type.index.title";

const label = (type: InternshipType) => type?.label?.pl;

export const InternshipTypeManagement = () => {
    const { t } = useTranslation("management");
    const [result, setTypesPromise] = useAsyncState<InternshipType[]>();
    const [selected, setSelected] = useState<InternshipType[]>([]);
    const spacing = useSpacing(2);

    const updateTypeList = () => {
        setTypesPromise(api.type.all());
    }

    const handleTypeDelete = async (type: OneOrMany<InternshipType>) => {
        await api.type.remove(type);
        updateTypeList();
    }

    useEffect(updateTypeList, []);

    const DeleteTypeAction = createDeleteAction({ label, onDelete: handleTypeDelete });

    const columns: Column<InternshipType>[] = [
        {
            field: "id",
            title: "ID",
            width: 0,
            defaultSort: "asc",
            filtering: false,
        },
        {
            title: t("type.field.label"),
            render: type => <MultilingualCell value={ type.label }/>,
            customSort: fieldComparator("label", multilingualStringComparator),
        },
        {
            title: t("type.field.description"),
            render: type => type.description && <MultilingualCell value={ type.description }/>,
            sorting: false,
        },
        {
            title: t("type.field.flags"),
            render: type => <>
                { type.requiresDeanApproval && <Tooltip title={ t("type.flag.dean-approval") as string }><AccountCheck/></Tooltip> }
                { type.requiresInsurance && <Tooltip title={ t("type.flag.insurance") as string }><ShieldCheck/></Tooltip> }
            </>,
            width: 0,
            filtering: true,
            sorting: false,
        },
        actionsColumn(type => <>
            <DeleteTypeAction resource={ type }/>
        </>)
    ];

    return <Page>
        <Page.Header maxWidth="lg">
            <Management.Breadcrumbs>
                <Typography color="textPrimary">{ t(title) }</Typography>
            </Management.Breadcrumbs>
            <Page.Title>{ t(title) }</Page.Title>
        </Page.Header>
        <Container maxWidth="lg" className={ spacing.vertical }>
            <Actions>
                <Button onClick={ updateTypeList } startIcon={ <Refresh /> }>{ t("refresh") }</Button>
            </Actions>
            { selected.length > 0 && <BulkActions>
                <DeleteTypeAction resource={ selected }>
                    { action => <Button startIcon={ <Delete /> } onClick={ action }>{ t("actions.delete") }</Button> }
                </DeleteTypeAction>
            </BulkActions> }
            <Async async={ result } keepValue>{
                pages => <MaterialTable
                    title={ <MaterialTableTitle result={ result } label={ t(title) }/> }
                    columns={ columns }
                    data={ pages }
                    onSelectionChange={ pages => setSelected(pages) }
                    options={ { selection: true, pageSize: 10 } }
                />
            }</Async>
        </Container>
    </Page>
}
